@@ -1,5 +1,58 @@
+#' @title Evaluate fitted taper curve
+#' @description This is the actual function to estimate diameters according to
+#' the fitted mixed B-splines model.
+#' @param xm relative heights for which measurements are available
+#' @param ym corresponding diameter measurements in height \code{xm}
+#' @param xp relative heights for which predictions are required
+#' @param par.lme Fitted model object, return of \code{\link{TapeR_FIT_LME.f}}
+#' @param Rfn list with function name to provide estimated or assumed residual 
+#' variances for the given measurements, optionally parameters for such functions
+#' @param ... not currently used
+#' @details This function is the actual working horse for prediction using the
+#' fitted taper model. Based on the model \code{par.lme} and the measured
+#' diameters \code{ym} and corresponding (relative) heights \code{xm} of a 
+#' specific tree (there might be just one measurement), the random
+#' effect parameters and subsequently diameters are estimated. Depending on the
+#' parameter \code{Rfn}, the calibrated taper curve is forced through the
+#' given diameter \code{ym} (\code{Rfn = list(fn="zero")}), or calibrated using
+#' the complete residual variance-covariance information 
+#' (\code{Rfn = list(fn="sig2")}, the default). 
+#' Further assumptions are possible, see also \code{\link{resVar}} and
+#' Kublin et al. (2013) p. 987 for more details.
+#' @return a list holding nine elements:
+#' \itemize{
+#'   \item b_fix fixed effects parameter of taper model
+#'   \item b_rnd random effects parameter given tree (posterior mean b_k)
+#'   \item yp estimated diameter in height \code{xp}
+#'   \item KOV_Mean variance-covariance matrix of expected value
+#'   \item KOV_Pred variance-covariance matrix of prediction
+#'   \item CI_Mean mean and limits of confidence interval
+#'   \item MSE_Mean mean squared error of expected value
+#'   \item MSE_Pred mean squared error of prediction
+#'   \item CI_Pred mean and limits of prediction interval
+#' }
+#' @author Edgar Kublin
+#' @seealso \code{\link{E_DHx_HmDm_HT.f}}, \code{\link{E_VOL_AB_HmDm_HT.f}}, 
+#' \code{\link{resVar}}
+#' @importFrom stats qt
+#' @examples
+#' data("SK.par.lme")
+#' TapeR:::SK_EBLUP_LME.f(1.3/27, 30, 1.3/27, SK.par.lme)
+#' ## using empirical best linear unbiased estimator: estimate != 30
+#' TapeR:::SK_EBLUP_LME.f(1.3/27, 30, 1.3/27, SK.par.lme, Rfn=list(fn="sig2"))$yp
+#' ## interpolate / force through given diameter: estimate  == 30
+#' TapeR:::SK_EBLUP_LME.f(1.3/27, 30, 1.3/27, SK.par.lme, Rfn=list(fn="zero"))$yp
+#' TapeR:::SK_EBLUP_LME.f(1.3/27, 30, c(1.3, 5)/27, SK.par.lme)
+#' par.lme <- SK.par.lme
+#' h <- 12 # tree height
+#' xm <- c(1.3, 3) / h # relative measuring height
+#' ym <- c(8, 7.5) # measured diameter
+#' xp <- c(0.5, 1) / h # relative prediction height
+#' TapeR:::SK_EBLUP_LME.f(xm, ym, xp, SK.par.lme)
+#' @export
+
 SK_EBLUP_LME.f <-
-function(xm, ym, xp, par.lme, ...){
+function(xm, ym, xp, par.lme, Rfn=list(fn="sig2"), ...){
 #   ************************************************************************************************
 
 	#	xm = xm_i; ym = ym_i; xp = xp_i; par.lme = SK_FIT_LME$par.lme
@@ -9,10 +62,10 @@ function(xm, ym, xp, par.lme, ...){
 	#   Design Matrizen X und Z zu den Kalibrierungsdaten :.........................................
 
 		x_k 	= xm
-		y_k     = ym
+		y_k   = ym
 
-		X_k = XZ_BSPLINE.f(x_k, par.lme)$X
-		Z_k = XZ_BSPLINE.f(x_k, par.lme)$Z
+		X_k = XZ_BSPLINE.f(x_k, par.lme$knt_x, par.lme$ord_x)
+		Z_k = XZ_BSPLINE.f(x_k, par.lme$knt_z, par.lme$ord_z)
 
 	#   Feste Effekte - Mittlere Schaftkurve in der GesamtPopulation (PA) : E[y|x] = X*b_fix:.......
 
@@ -28,14 +81,12 @@ function(xm, ym, xp, par.lme, ...){
 
 		sig2_eps = par.lme$sig2_eps
 		dfRes    = par.lme$dfRes
-
-		R_k = diag(sig2_eps,ncol(Z_KOVb_Zt_k))
-	#	R_k = diag(0,ncol(Z_KOVb_Zt_k))                 			#	Interpolation der Messwerte
-
+    
+	  rv <- resVar(x_k, fn = Rfn$fn, sig2 = sig2_eps, par = Rfn$par)
+	  R_k = diag(rv, ncol(Z_KOVb_Zt_k))
+	  
 	#   Kovarianzmatrix der Beobachtungen (Sigma.i) :...............................................
 
-	#	rm(KOV_y_k, KOVinv_y_k)
-
 		KOV_y_k		= Z_KOVb_Zt_k + R_k                       	#   SIGMA_k in V&C(1997)(6.2.3)
 		KOVinv_y_k  = solve(KOV_y_k);                    		#   SIGMA_k^(-1)
 
@@ -49,8 +100,8 @@ function(xm, ym, xp, par.lme, ...){
 	#	x_pre 	= xp[order(xp)]
 		x_pre 	= xp
 
-		X_kh = XZ_BSPLINE.f(x_pre, par.lme)$X
-		Z_kh = XZ_BSPLINE.f(x_pre, par.lme)$Z
+		X_kh = XZ_BSPLINE.f(x_pre, par.lme$knt_x, par.lme$ord_x)
+		Z_kh = XZ_BSPLINE.f(x_pre, par.lme$knt_z, par.lme$ord_z)
 
 	#   ********************************************************************************************
 		yp = EBLUP_y_kh  = X_kh%*%b_fix + Z_kh%*%EBLUP_b_k    		#	V&C(1997) (6.2.52)
